package my.petstore.entity;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

@Entity
public class OrderData extends BaseEntity {

    private OrderData() {}
    public OrderData(Cart cart) {
        orderDate = new Date();

        creditCard = "999 9999 9999 9999";
        expiryDate = "12/03";
        cardType = "Visa";
        setStatus(new OrderStatus(new Date(), "P", this));
        
        if (cart != null) {
            totalPrice = cart.getTotal();
            for (CartItem cartItem : cart.getAllCartItems()) {
                addLineItem(cartItem);
            }
        }
    }

    @Temporal(TemporalType.DATE)
    private Date orderDate;
    private int totalPrice;
    @Column(name = "shipaddr")
    private String shipAddress;
    private String shipToName;
    private String creditCard;
    @Column(name = "exprdate")
    private String expiryDate;
    private String cardType;

    @OneToOne(cascade = CascadeType.ALL, mappedBy = "orderData", fetch = FetchType.EAGER)
    private OrderStatus status;

    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JoinColumn(name = "order_id", nullable = false)
    private List<LineItem> lineItems = new ArrayList<LineItem>();

    public Date getOrderDate() { return orderDate; }
    public void setOrderDate(Date orderDate) { this.orderDate = orderDate; }

    public int getTotalPrice() { return totalPrice; }
    public void setTotalPrice(int totalPrice) { this.totalPrice = totalPrice; }

    public String getShipAddress() { return shipAddress; }
    public void setShipAddress(String shipAddress) { this.shipAddress = shipAddress; }

    public String getShipToName() { return shipToName; }
    public void setShipToName(String shipToName) { this.shipToName = shipToName; }

    public String getCreditCard() { return creditCard; }
    public void setCreditCard(String creditCard) { this.creditCard = creditCard; }

    public String getExpiryDate() { return expiryDate; }
    public void setExpiryDate(String expiryDate) { this.expiryDate = expiryDate; }

    public String getCardType() { return cardType; }
    public void setCardType(String cardType) { this.cardType = cardType; }

    public OrderStatus getStatus() { return status; }
    public void setStatus(OrderStatus status) { this.status = status; }


    public List<LineItem> getLineItems() { return lineItems; }
    public void setLineItems(List<LineItem> lineItems) { this.lineItems = lineItems; }

    private void addLineItem(CartItem cartItem) {
        LineItem lineItem = new LineItem(lineItems.size() + 1, cartItem);
        addLineItem(lineItem);
    }

    private void addLineItem(LineItem lineItem) {
        lineItems.add(lineItem);
    }

    public static final OrderData NULL = new OrderData() {
        {
            setOrderDate(new Date());
            setShipAddress("");
            setShipToName("");
            setCreditCard("");
            setExpiryDate("");
            setCardType("");
            setStatus(new OrderStatus(null, null, this));
        }
    };
}
