package my.petstore.service;

import java.util.List;
import java.util.Collections;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import my.petstore.entity.Category;
import my.petstore.entity.Item;
import my.petstore.entity.Product;

import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


@Service
@Repository
@Transactional(readOnly = true)
public class ProductServiceImpl implements ProductService {

    @PersistenceContext
    private EntityManager em;

    public List<Category> getCategoryList() {
        return this.em.createQuery("SELECT cat FROM Category cat").getResultList();
    }

    public Category getCategory(Integer categoryId) {
        Category category = this.em.find(Category.class, categoryId);
        return category != null ? category : Category.NULL;
    }

    public List<Product> searchProductList(String keyword) {
        if (keyword == null || keyword.isEmpty()) {
            return Collections.<Product>emptyList();
        }
        Query query = this.em.createQuery("SELECT product FROM Product product WHERE lower(product.name) LIKE :keyword");
        query.setParameter("keyword", keyword.toLowerCase() + "%");
        return query.getResultList();
    }

    public Product getProduct(Integer productId) {
        Product product = this.em.find(Product.class, productId);
        return product != null ? product : Product.NULL;
    }

    public Item getItem(Integer itemId) {
        Item item = this.em.find(Item.class, itemId);
        return item != null ? item : Item.NULL;
    }

    public boolean isItemInStock(Integer itemId) {
        return this.em.find(Item.class, itemId).getInventory().getQuantity() > 0;
    }
}
