package my.petstore.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import my.petstore.entity.Cart;
import my.petstore.entity.OrderData;
import my.petstore.service.OrderService;
import my.petstore.validation.OrderValidator;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

@Controller
@SessionAttributes("orderData")
@RequestMapping(value="/order")
public class OrderFormController {

    @Autowired
    private OrderService orderService;

    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        binder.setValidator(new OrderValidator());
    }

    @ModelAttribute()
    public void createFormBean(HttpServletRequest request, @RequestParam(required=false) Integer page, Model model) {
        if (page == null) {
            Cart cart = (Cart) request.getSession().getAttribute("sessionCart");
            if (cart == null) {
                throw new RuntimeException("invalid access");
            }
            model.addAttribute(new OrderData(cart));

            page = 1;
            List<String> creditCardTypes = new ArrayList<String>();
            creditCardTypes.add("Visa");
            creditCardTypes.add("MasterCard");
            creditCardTypes.add("American Express");
            model.addAttribute("creditCardTypes", creditCardTypes);
        } else {
            page++;
        }
        model.addAttribute("page", page);
    }

    @RequestMapping(method=RequestMethod.GET)
    public String formNewOrder() {
        return "NewOrderForm";
    }

    @RequestMapping(method=RequestMethod.POST, params="page=1")
    public String formShipping(OrderData orderData, @RequestParam(required=true) Integer page) {
        return "ShippingForm";
    }

    @RequestMapping(method=RequestMethod.POST, params="page=2")
    public String formConfirm(OrderData orderData, @RequestParam(required=true) Integer page) {
        return "ConfirmOrder";
    }

    @RequestMapping(method=RequestMethod.POST, params="page=3")
    public String formFinish(@Valid OrderData orderData, BindingResult result, @RequestParam(required=true) Integer page, HttpSession session, Model model) {
        if (result.hasErrors()) {
            model.addAttribute("page", 1);
            return "NewOrderForm";
        }

        try {
            orderService.storeOrderData(orderData);
        } catch (OrderService.OutOfOrderException ex) {
            result.rejectValue("lineItems", "outOfOrder", "Out of order");
            model.addAttribute("page", 1);
            return "NewOrderForm";
        }

        session.removeAttribute("sessionCart");
        model.addAttribute("order", orderData);
        model.addAttribute("message", "Thank you, your order has been submitted.");
        return "ViewOrder";
    }
}
